%% we are simulating a sinc RF pulse with max B1 amplitude of 20uT and phase of 0 (along X' axis)
%% we are simulating this for a range of off-resonance frequencies

clear all;
close all;
gamma=42.58; %%42.58 Hz/uT
B1_max=3; %% max B1 maplitude in uT

freq=[-2000:1:1999]; %% we are simulating plus or minus 2000 Hz range of off-resonance frequencies
time_ticks=[-500:1:499]; %% these are time ticks, each time tick corresponds to 1us, so the entire pulse is 1ms long
%B1=B1_max*sin(time_ticks/250*pi)./(time_ticks/250*pi); B1(501)=B1_max; %% this is the vector storing the actual B1 amplitude at any time point (at 1us intervals) during the RF pulse
B1=B1_max*ones(1,length(time_ticks));
figure, plot(time_ticks,B1), title("Envelope B1 field as a function of time"); %%the actual B1 profile of the RF pulse

%% now we need to apply the Bloch equation to solve the forced precession for off-resonance conditions

M=zeros(3,length(time_ticks)+1,4000);%% M will store the initial M vector and the M vector after each of the 1000 times points during the RF pulse, all these for each off-resonance frequency
for j=1:4000  %% we are considering each of the off-resonance frequencies in increments of 1Hz
    fq=freq(j);%%frequency in Hz
    B_eff=[B1; zeros(1,length(B1)); fq*ones(1,length(B1))/gamma]; 
    M(:,1,j)=[0;0;1];%%initialization of the magnetizations. Every spin, regardless of off-resonance, is initialized to 1 at the beginning of the RF pulse
    for k=1:length(B1)
        rot_axis=squeeze(B_eff(:,k));
        axis_len=sqrt(rot_axis(1)^2+rot_axis(3)^2);
        angle=atan(rot_axis(3)/rot_axis(1))*180/pi;
        if rot_axis(1)<0
            angle=angle+180;
        end
        temp=Ry(-angle,squeeze(M(:,k,j)));
        temp=Rx(axis_len*gamma*360/1e6*2,temp);
        M(:,k+1,j)=Ry(angle,temp);
    end
   
end

M_endRF=squeeze(M(:,1001,:));
figure, plot(freq,sqrt(M_endRF(1,:).^2+M_endRF(2,:).^2)), title("Final Mxy as a function of off-resonance freq"),axis([-2000 2000 0 1]);

M_onres=squeeze(M(:,2:end,2001));
figure, plot(time_ticks,sqrt(M_onres(1,:).^2+M_onres(2,:).^2)), axis([-500 499 0 1]), title("Evolution of Mxy during RF pulse for on-resonance spins"); 